@@ -46,7 +46,7 @@
 	 GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS)
 
 #define NAUTILUS_FILE_DEFAULT_ATTRIBUTES				\
-	"standard::*,access::*,mountable::*,time::*,unix::*,owner::*,selinux::*,thumbnail::*,mountable::*"
+	"standard::*,access::*,mountable::*,time::*,unix::*,owner::*,selinux::*,thumbnail::*,id::filesystem,trash::orig-path"
 
 /* These are in the typical sort order. Known things come first, then
  * things where we can't know, finally things where we don't yet know.
@@ -90,7 +90,8 @@ struct NautilusFileDetails
 	
 	eel_ref_str mime_type;
 	
-	char* selinux_context;
+	char *selinux_context;
+	char *description;
 	
 	GError *get_info_error;
 	
@@ -114,6 +115,13 @@ struct NautilusFileDetails
 	char *custom_icon;
 	GFile *activation_location;
 
+	/* used during DND, for checking whether source and destination are on
+	 * the same file system.
+	 */
+	eel_ref_str filesystem_id;
+
+	char *trash_orig_path;
+
 	/* The following is for file operations in progress. Since
 	 * there are normally only a few of these, we can move them to
 	 * a separate hash table or something if required to keep the
@@ -190,6 +198,7 @@ struct NautilusFileDetails
 	eel_boolean_bit has_open_window               : 1;
 
 	eel_boolean_bit is_launcher                   : 1;
+	eel_boolean_bit is_foreign_link               : 1;
 	eel_boolean_bit is_symlink                    : 1;
 	eel_boolean_bit is_mountpoint                 : 1;
 	eel_boolean_bit is_hidden                     : 1;
