@@ -330,6 +330,14 @@ void                    nautilus_file_set_integer_metadata              (Nautilu
 									 int                             default_metadata,
 									 int                             metadata);
 
+#define UNDEFINED_TIME ((time_t) (-1))
+
+time_t                  nautilus_file_get_time_metadata                 (NautilusFile                  *file,
+									 const char                    *key);
+void                    nautilus_file_set_time_metadata                 (NautilusFile                  *file,
+									 const char                    *key,
+									 time_t                         time);
+
 
 /* Attributes for file objects as user-displayable strings. */
 char *                  nautilus_file_get_string_attribute              (NautilusFile                   *file,
@@ -369,6 +377,8 @@ int                     nautilus_file_compare_for_sort_by_attribute_q   (Nautilu
 									 GQuark                          attribute,
 									 gboolean                        directories_first,
 									 gboolean                        reversed);
+gboolean                nautilus_file_is_date_sort_attribute_q          (GQuark                          attribute);
+
 int                     nautilus_file_compare_display_name              (NautilusFile                   *file_1,
 									 const char                     *pattern);
 
@@ -377,7 +387,8 @@ gboolean                nautilus_file_is_hidden_file                    (Nautilu
 gboolean                nautilus_file_is_backup_file                    (NautilusFile                   *file);
 gboolean                nautilus_file_should_show                       (NautilusFile                   *file,
 									 gboolean                        show_hidden,
-									 gboolean                        show_backup);
+									 gboolean                        show_backup,
+									 gboolean                        show_foreign);
 GList                  *nautilus_file_list_filter_hidden_and_backup     (GList                          *files,
 									 gboolean                        show_hidden,
 									 gboolean                        show_backup);
@@ -387,6 +398,7 @@ GList                  *nautilus_file_list_filter_hidden_and_backup     (GList
  * Getting this can require reading the contents of the file.
  */
 gboolean                nautilus_file_is_launcher                       (NautilusFile                   *file);
+gboolean                nautilus_file_is_foreign_link                   (NautilusFile                   *file);
 gboolean                nautilus_file_has_activation_uri                (NautilusFile                   *file);
 char *                  nautilus_file_get_activation_uri                (NautilusFile                   *file);
 GFile *                 nautilus_file_get_activation_location           (NautilusFile                   *file);
