@@ -42,7 +42,6 @@
 #include <fcntl.h>
 #include <gdk/gdkx.h>
 #include <glib/gi18n.h>
-#include <libgnome/gnome-util.h>
 #include <libnautilus-private/nautilus-desktop-icon-file.h>
 #include <libnautilus-private/nautilus-directory-background.h>
 #include <libnautilus-private/nautilus-directory-notify.h>
@@ -546,6 +545,7 @@ fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 	
 	nautilus_icon_container_set_is_fixed_size (icon_container, TRUE);
 	nautilus_icon_container_set_is_desktop (icon_container, TRUE);
+	nautilus_icon_container_set_store_layout_timestamps (icon_container, TRUE);
 
 	/* Set allocation to be at 0, 0 */
 	allocation = &GTK_WIDGET (icon_container)->allocation;
@@ -562,6 +562,9 @@ fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 
 	fm_directory_view_ignore_hidden_file_preferences
 		(FM_DIRECTORY_VIEW (desktop_icon_view));
+
+	fm_directory_view_set_show_foreign (FM_DIRECTORY_VIEW (desktop_icon_view),
+			                    FALSE);
 	
 	/* Set our default layout mode */
 	nautilus_icon_container_set_layout_mode (icon_container,
@@ -599,9 +602,9 @@ action_new_launcher_callback (GtkAction *action, gpointer data)
 
 	nautilus_launch_application_from_command (gtk_widget_get_screen (GTK_WIDGET (data)),
 						  "gnome-desktop-item-edit", 
-						  "gnome-desktop-item-edit --create-new",
-						  desktop_directory, 
-						  FALSE);
+						  "gnome-desktop-item-edit",
+						  FALSE,
+						  "--create-new", desktop_directory, NULL);
 	g_free (desktop_directory);
 
 }
@@ -615,8 +618,8 @@ action_change_background_callback (GtkAction *action,
 	nautilus_launch_application_from_command (gtk_widget_get_screen (GTK_WIDGET (data)),
 						  _("Background"),
 						  "gnome-appearance-properties",
-						  "--show-page=background",
-						  FALSE);
+						  FALSE,
+						  "--show-page=background", NULL);
 }
 
 static void
